@@ -1,4 +1,3 @@
-import pandas as pd
 from datetime import timedelta
 from bonsai_cli.commands.diaglets.diaglet_base import Diaglet
 from bonsai_cli.commands.diaglets.diaglet_configuration import DiagletConfiguration
@@ -31,7 +30,7 @@ class SDKVersionDiaglet(Diaglet):
         if self.diagnostic_configuration.is_test:
             # Create a Python list of dictionaries
             data = [{"Message": "This is an automated test"}]
-            df = pd.DataFrame(data)
+            df = self.get_test_dataframe(data)
         else:
             df, _ = self.run_kql_query(kql_query, timespan)
 
@@ -39,4 +38,4 @@ class SDKVersionDiaglet(Diaglet):
             # just print what the message says
             self.message = df.iloc[0]["Message"]
         else:
-            self.message = f"Could not retrieve SDK version details."
+            self.message = f"Could not determine SDK version."
